const char* rtst_checkeq_rcsid = "$Header: /u/drspeech/repos/rtst/rtst_checkeq.c,v 1.20 1999/03/03 23:47:39 dpwe Exp $";
/*
**
** rtst - various routines useful for writing test programs
**
** This file contains routines for checking stuff
**
** Note - this is used on T0 simulators, hence efficiency is important,
** and so we avoid using printf wherever possible.
*/

#include <stdlib.h>
#include <assert.h>
#include "rtst.h"

static int
isnan_f_b(float f)
{
    union {
	float f;
	rtst_uint32 u;
    } v;
    rtst_uint32 u;

    v.f = f;
    u = v.u;

    if ((u & 0x7f800000u)==0x7f800000u && (u&0x007fffff)!=0)
	return(RTST_TRUE);
    else
	return(RTST_FALSE);
}

static int
isnan_d_b(double d)
{
    union {
	double d;
	struct
	{
	    rtst_uint32 u1;
	    rtst_uint32 u2;
	} u;
    } v;
    rtst_uint32 u1;
    rtst_uint32 u2;

    v.d = d;
#ifdef WORDS_BIGENDIAN
    u1 = v.u.u1;
    u2 = v.u.u2;
#else /* !WORDS_BIGENDIAN */
    /* bytes-in-int take care of themselves, but have to swap lwords */
    u1 = v.u.u2;
    u2 = v.u.u1;
#endif

    if ((u1 & 0x7ff00000u)==0x7ff00000u &&
	((u1&0x000fffff)!=0 || (u2!=0)) )
	return(RTST_TRUE);
    else
	return(RTST_FALSE);
}


void
_rtst_checkeq_vi8vi8(size_t len,
		     const rtst_int8* a,
		     const rtst_int8* b,
		     const char* file,
		     unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i])
	    rtst_failed("%s:%d: rtst_checkeq_vi8vi8 failed, "
			    "a[%i]=%i b[%i]=%i.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vu8vu8(size_t len,
		     const rtst_uint8* a,
		     const rtst_uint8* b,
		     const char* file,
		     unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i])
	    rtst_failed("%s:%d: rtst_checkeq_vu8vu8 failed, "
			    "a[%i]=%i b[%i]=%i.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vi16vi16(size_t len,
                       const rtst_int16* a,
                       const rtst_int16* b,
                       const char* file,
                       unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i])
	    rtst_failed("%s:%d: rtst_checkeq_vi16vi16 failed, "
			    "a[%i]=%i b[%i]=%i.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vu16vu16(size_t len,
                       const rtst_uint16* a,
                       const rtst_uint16* b,
                       const char* file,
                       unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i])
	    rtst_failed("%s:%d: rtst_checkeq_vu16vu16 failed, "
			    "a[%i]=%i b[%i]=%i.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vi32vi32(size_t len,
                       const rtst_int32* a,
                       const rtst_int32* b,
                       const char* file,
                       unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i])
	    rtst_failed("%s:%d: rtst_checkeq_vi32vi32 failed, "
			    "a[%i]=%i b[%i]=%i.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vu32vu32(size_t len,
                       const rtst_uint32* a,
                       const rtst_uint32* b,
                       const char* file,
                       unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i])
	    rtst_failed("%s:%d: rtst_checkeq_vu32vu32 failed, "
			    "a[%i]=%u b[%i]=%u.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vfvf(size_t len, const float* a, const float* b,
		      const char* file, unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i] && !(isnan_f_b(a[i]) && isnan_f_b(b[i])))
	    rtst_failed("%s:%d: rtst_checkeq_vfvf failed, "
			    "a[%i]=%g b[%i]=%g.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_vdvd(size_t len, const double* a, const double* b,
		      const char* file, unsigned int line)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	if (a[i]!=b[i] && !(isnan_d_b(a[i]) && isnan_d_b(b[i])))
	    rtst_failed("%s:%d: rtst_checkeq_vdvd failed, "
			    "a[%i]=%g b[%i]=%g.\n",
			    file, line, i, a[i], i, b[i]);
    }
}

void
_rtst_checkeq_mi8mi8(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const rtst_int8* a, const rtst_int8* b,
		   const char* file, unsigned int line)
{
    size_t i, j;

    for (i=0; i<rows; i++)
    {
	for (j=0; j<cols; j++)
	{
	    if (a[j]!=b[j])
	    {
		rtst_failed("%s:%d: rtst_checkeq_mi8mi8 failed, "
			    "a[%lu,%lu]=%d b[%lu,%lu]=%d.\n",
			    file, line,
			    (unsigned long) i, (unsigned long) j, a[j],
			    (unsigned long) i, (unsigned long) j, b[j]);
	    }
	}
	a += a_lda;
	b += b_lda;
    }
}

void
_rtst_checkeq_mi16mi16(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const rtst_int16* a, const rtst_int16* b,
		   const char* file, unsigned int line)
{
    size_t i, j;

    for (i=0; i<rows; i++)
    {
	for (j=0; j<cols; j++)
	{
	    if (a[j]!=b[j])
	    {
		rtst_failed("%s:%d: rtst_checkeq_mi16mi16 failed, "
			    "a[%lu,%lu]=%d b[%lu,%lu]=%d.\n",
			    file, line,
			    (unsigned long) i, (unsigned long) j, a[j],
			    (unsigned long) i, (unsigned long) j, b[j]);
	    }
	}
	a += a_lda;
	b += b_lda;
    }
}

void
_rtst_checkeq_mi32mi32(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const rtst_int32* a, const rtst_int32* b,
		   const char* file, unsigned int line)
{
    size_t i, j;

    for (i=0; i<rows; i++)
    {
	for (j=0; j<cols; j++)
	{
	    if (a[j]!=b[j])
	    {
		rtst_failed("%s:%d: rtst_checkeq_mi32mi32 failed, "
			    "a[%lu,%lu]=%d b[%lu,%lu]=%d.\n",
			    file, line,
			    (unsigned long) i, (unsigned long) j, a[j],
			    (unsigned long) i, (unsigned long) j, b[j]);
	    }
	}
	a += a_lda;
	b += b_lda;
    }
}

void
_rtst_checkeq_mfmf(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const float* a, const float* b,
		   const char* file, unsigned int line)
{
    size_t i, j;

    for (i=0; i<rows; i++)
    {
	for (j=0; j<cols; j++)
	{
	    if (a[j]!=b[j] && !(isnan_f_b(a[j]) && isnan_f_b(b[j])))
	    {
		rtst_failed("%s:%d: rtst_checkeq_mfmf failed, "
			    "a[%lu,%lu]=%g b[%lu,%lu]=%g.\n",
			    file, line,
			    (unsigned long) i, (unsigned long) j, a[j],
			    (unsigned long) i, (unsigned long) j, b[j]);
	    }
	}
	a += a_lda;
	b += b_lda;
    }
}

void
_rtst_checkeq_mdmd(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const double* a, const double* b,
		   const char* file, unsigned int line)
{
    size_t i, j;

    for (i=0; i<rows; i++)
    {
	for (j=0; j<cols; j++)
	{
	    if (a[j]!=b[j] && !(isnan_f_b(a[j]) && isnan_f_b(b[j])))
	    {
		rtst_failed("%s:%d: rtst_checkeq_mdmd failed, "
			    "a[%lu,%lu]=%g b[%lu,%lu]=%g.\n",
			    file, line,
			    (unsigned long) i, (unsigned long) j, a[j],
			    (unsigned long) i, (unsigned long) j, b[j]);
	    }
	}
	a += a_lda;
	b += b_lda;
    }
}

/* Use local version of abs to avoid linking in math library. */
static float
abs_f_f(float x)
{
    if (x < 0.0f)
        return -x;
    else
        return x;
}

void
_rtst_checknear_fvfvf(size_t len, float tolerance,
		      const float* a, const float* b,
		      const char* file, unsigned int line)
{
    size_t i;

    assert(tolerance>=0.0);
    for (i=0; i<len; i++)
    {
        const float a_val = a[i];
        const float b_val = b[i];

	/* Two NaNs are considered the same */
	if (!(isnan_f_b(a_val) && isnan_f_b(b_val)))
	{
	    if (abs_f_f(a_val - b_val) > tolerance)
		rtst_failed("%s:%d: rtst_checknear_fvfvf failed, "
			    "a[%i]=%g b[%i]=%g tol=%g.\n",
			    file, line, i, a_val, i, b_val, tolerance);
	}
    }
}

/* Use local version of abs to avoid linking in math library. */
static double
abs_d_d(double x)
{
    if (x < 0.0)
        return -x;
    else
        return x;
}

void
_rtst_checknear_dvdvd(size_t len, double tolerance,
		      const double* a, const double* b,
		      const char* file, unsigned int line)
{
    size_t i;

    assert(tolerance>=0.0);
    for (i=0; i<len; i++)
    {
        const double a_val = a[i];
        const double b_val = b[i];

	/* Two NaNs are considered the same */
	if (!(isnan_d_b(a_val) && isnan_d_b(b_val)))
	{
	    if (abs_d_d(a_val-b_val) > tolerance)
		rtst_failed("%s:%d: rtst_checknear_dvdvd failed, "
			    "a[%i]=%g b[%i]=%g tol=%g.\n",
			    file, line, i, a_val, i, b_val, tolerance);
	}
    }
}


void
_rtst_checkrange_i16i16vi16(size_t len, rtst_int16 min, rtst_int16 max,
			    const rtst_int16* in,
			    const char* file, unsigned int line)
{
    size_t i;
    
    for (i=0; i<len; i++)
    {
	if (in[i]<min || in[i]>max)
	{
		rtst_failed("%s:%d: rtst_checkrange_i16i16vi16 failed, "
			    "min=%lu max=%lu in[%i]=%lu.\n",
			    file, line, (unsigned long) min,
			    (unsigned long) max, i, (unsigned long) in[i]);
	}
    }
}

void
_rtst_checkrange_i32i32vi32(size_t len, rtst_int32 min, rtst_int32 max,
			    const rtst_int32* in,
			    const char* file, unsigned int line)
{
    size_t i;
    
    for (i=0; i<len; i++)
    {
	if (in[i]<min || in[i]>max)
	{
		rtst_failed("%s:%d: rtst_checkrange_i32i32vi32 failed, "
			    "min=%lu max=%lu in[%i]=%lu.\n",
			    file, line, (unsigned long) min,
			    (unsigned long) max, i, (unsigned long) in[i]);
	}
    }
}

void
_rtst_checkrange_ffvf(size_t len, float min, float max, const float* in,
		      const char* file, unsigned int line)
{
    size_t i;
    
    for (i=0; i<len; i++)
    {
	if (in[i]<min || in[i]>max || isnan_f_b(in[i]))
	{
		rtst_failed("%s:%d: rtst_checkrange_ffvf failed, "
			    "min=%g max=%g in[%i]=%g.\n",
			    file, line, min, max, i, in[i]);
	}
    }
}

void
_rtst_checkrange_ddvd(size_t len, double min, double max, double* in,
		      const char* file, unsigned int line)
{
    size_t i;
    
    for (i=0; i<len; i++)
    {
	if (in[i]<min || in[i]>max || isnan_d_b(in[i]))
	{
		rtst_failed("%s:%d: rtst_checkrange_ddvd failed, "
			    "min=%g max=%g in[%i]=%g.\n",
			    file, line, min, max, i, in[i]);
	}
    }
}

